<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @package    mod_iprsmart
 * @subpackage backup-moodle2
 * @copyright 2022 Yurii Senin
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Define all the restore steps that will be used by the restore_iprsmart_activity_task
 */

/**
 * Structure step to restore one iprsmart activity
 */
class restore_iprsmart_activity_structure_step extends restore_activity_structure_step {

    protected function define_structure() {
        $paths = array();
        $paths[] = new restore_path_element('iprsmart', '/activity/iprsmart');


        // Return the paths wrapped into standard activity structure
        return $this->prepare_activity_structure($paths);
    }

    public function process_iprsmart($data) {
        global $DB;
        $data = (object)$data;
        $oldid = $data->id;
        $data->course = $this->get_courseid();
        // Any changes to the list of dates that needs to be rolled should be same during course restore and course reset.
        // See MDL-9367.

        // insert the iprsmart record
        $newitemid = $DB->insert_record('iprsmart', $data);
        // immediately after inserting "activity" record, call this
        $this->apply_activity_instance($newitemid);
    }

    protected function after_execute(): void
    {


        // Add url related files, no need to match by itemname (just internally handled context)
        $this->add_related_files('mod_iprsmart', 'intro', null);
    }
}
